"""
xchange: Many cryptocurrency exchange APIs, a single and unified API client.
~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2019 by Martin Zugnoni.
:license: MIT License, see LICENSE for more details.
"""

__title__ = 'xchange'
__version__ = '0.0.6'
__build__ = 0x00001
__author__ = 'Martin Zugnoni'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018-2019 Martin Zugnoni'
